package game;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import main.*;

public class GamePanel extends JPanel implements Runnable
{
	Dimension size;   // パネルの大きさ
	MainPanel mp;
	Thread td;
	boolean in_game = true;
	Hero hr;   // 主人公
	Road rd;   // 背景（道）
	double up = 0;   // 上向き加速度
	double down = 0;   // 下向き加速度
	boolean jump = false;   // ジャンプ中か？
			// コンストラクタ
	public GamePanel(Dimension size1, MainPanel mp1)
	{
		size = size1;
		mp   = mp1;
					// レイアウトマネージャの停止
		setLayout(null);
					// 背景色の設定
		setBackground(Color.white);
					// 主人公と道の生成
		rd = new Road(size, mp);
		hr = new Hero(size, rd, mp);
					// スレッドの生成
		td = new Thread(this);
		td.start();
					// マウスリスナの追加
		addMouseListener(new Mouse());
	}
			// スレッドの実行
	public void run()
	{
		while (in_game) {
			try {
				td.sleep(30);
			}
			catch (InterruptedException e) {}
					// 移動と再描画
			hr.v_y += (up - down);
			hr.x   += (int)hr.v_x;
			hr.y   -= (int)hr.v_y;
			rd.x   += (int)rd.v_x;
			repaint();
					// 道の状態のチェック
			int x = (hr.x - rd.x + hr.width / 2) / rd.width;
							// ジャンプ中
			if (jump) {
				if (x >= rd.col) {
					in_game = false;
					mp.state = 3;   // ゲームオーバー
				}
				else {
					int y = (hr.y + hr.height) / rd.height;
					if (y >= 0) {
						if (y < rd.row && rd.blk[y][x] > 0) {
							jump   = false;
							down   = 0.0;
							hr.v_x = 1.0;
							hr.v_y = 0.0;
							hr.y   = size.height - rd.height * rd.r_no - hr.height;
							if (rd.blk[y][x] == 2) {
								in_game = false;
								mp.state = 2;   // ゲームクリア
							}
						}
						else if (y >= rd.row - rd.r_no) {
							in_game = false;
							mp.state = 3;   // ゲームオーバー
						}
					}
				}
			}
							// ジャンプ中でない
			else {
				int y = (hr.y + hr.height + rd.height / 2) / rd.height;
				if (x >= rd.col || rd.blk[y][x] == 0) {
					in_game = false;
					mp.state = 3;   // ゲームオーバー
				}
				else if (rd.blk[y][x] == 2) {
					in_game = false;
					mp.state = 2;   // ゲームクリア
				}
			}
		}
	}
			// 描画
	public void paintComponent(Graphics g)
	{
		super.paintComponent(g);   // 親クラスの描画
					// 主人公の描画
		g.drawImage(hr.image, hr.x, hr.y, this);
					// 背景の描画
		for (int i1 = 0; i1 < rd.col; i1++) {
			int x = rd.x + rd.width * i1;
			if (x + rd.width >= 0 && x <= size.width) {
				for (int i2 = 0; i2 < rd.row; i2++) {
					if (rd.blk[i2][i1] > 0) {
						int y = i2 * rd.height;
						g.drawImage(rd.image[rd.blk[i2][i1]-1], x, y, this);
					}
				}
			}
		}
	}
			// マウスイベントの処理
	class Mouse extends MouseAdapter {
					// ボタンが押されたとき
		public void mousePressed(MouseEvent e) {
			if (!jump) {
				up     = 1.0;
				down   = 0.5;
				hr.v_x = 2.0;
				jump   = true;
				hr.y--;   // 着地判定のため，1 ピクセルだけジャンプさせておく
			}
		}
					// ボタンが離されたとき
		public void mouseReleased(MouseEvent e) {
			up   = 0.0;
			down = 0.5;
		}
	}
}
