import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import java.util.Random;
import java.awt.geom.*;

public class Test {
	public static void main (String[] args)
	{
		Win win = new Win("円と三角形の衝突判定");
	}
}

class Win extends JFrame
{
	/******************/
	/* コンストラクタ */
	/******************/
	Win(String name)
	{
					// JFrameクラスのコンストラクタ（Windowのタイトルを引き渡す）
		super(name);
					// Windowの大きさ
		setSize(640, 470);   // +40, +70
					// ContentPane の取得と MainPanel の追加
		Dimension d  = getSize();   // Windowの大きさ
		d.width     -= 40;
		d.height    -= 70;
		MainPanel pn = new MainPanel(d);   // MainPanel オブジェクトの生成
		getContentPane().add(pn);   // MainPanel オブジェクトを ContentPane に追加
					// ウィンドウを表示
		setVisible(true);
					// イベントアダプタ
		addWindowListener(new WinEnd());
	}

	/******************************/
	/* 上，左，下，右の余白の設定 */
	/******************************/
	public Insets getInsets()
	{
		return new Insets(50, 20, 20, 20);
	}

	/************/
	/* 終了処理 */
	/************/
	class WinEnd extends WindowAdapter
	{
		public void windowClosing(WindowEvent e) {
			System.exit(0);
		}
	}
}

class MainPanel extends JPanel implements Runnable
{
	boolean state = true;
	double x;   // 物体の位置(x)
	double y;   // 物体の位置(y)
	double vx;   // 物体の速度(x)
	double vy;   // 物体の速度(y)
	double v = 20;   // 初期速度
	double dt = 0.1;   // 時間刻み幅
	int r = 20;   // 物体の半径
	Polygon tr;   // 三角形
	Ellipse2D.Double cir;   // 円
	Dimension d;   // パネルの幅と高さ
	double px[] = new double [15];   // 衝突判定位置(x)
	double py[] = new double [15];   // 衝突判定位置(y)
	Random rn;
	Thread th;
	boolean hit = false;
	int count = 0;

	MainPanel(Dimension d1)
	{
		d = d1;
		rn = new Random();
					// 初期設定
		ini_set();
		cir = new Ellipse2D.Double(x-r, y-r, 40, 40);

		double ang = Math.atan2(y-d.height/2, d.width/2-x);
		vx = v * Math.cos(ang);
		vy = -v * Math.sin(ang);
					// 三角形
		double cx = d.width / 2.0;
		double cy = d.height / 2.0;
		double x1 = cx;
		double y1 = cy - 50.0 / Math.cos(Math.PI/6);
		double x2 = cx - 50.0;
		double y2 = cy + 50.0 * Math.tan(Math.PI/6);
		double x3 = cx + 50.0;
		double y3 = cy + 50.0 * Math.tan(Math.PI/6);
		int xx[] = {(int)cx, (int)cx-50, (int)cx+50};
		int yy[] = {(int)y1, (int)y2, (int)y3};
		tr = new Polygon(xx, yy, 3);
					// 衝突判定位置
		int k = 0;
		px[k] = x1;
		py[k] = y1;
		k++;
		double sx = (x2 - x1) / 5;
		double sy = (y2 - y1) / 5;
		for (int i1 = 1; i1 < 5; i1++) {
			px[k] = x1 + i1 * sx;
			py[k] = y1 + i1 * sy;
			k++;
		}
		px[k] = x2;
		py[k] = y2;
		k++;
		sx = (x3 - x2) / 5;
		sy = (y3 - y2) / 5;
		for (int i1 = 1; i1 < 5; i1++) {
			px[k] = x2 + i1 * sx;
			py[k] = y2 + i1 * sy;
			k++;
		}
		px[k] = x3;
		py[k] = y3;
		k++;
		sx = (x1 - x3) / 5;
		sy = (y1 - y3) / 5;
		for (int i1 = 1; i1 < 5; i1++) {
			px[k] = x3 + i1 * sx;
			py[k] = y3 + i1 * sy;
			k++;
		}

		setBackground(new Color(238, 255, 238));   // 背景色の設定
		th = new Thread(this);   // スレッドの生成とスタート
		th.start();
	}
					// 初期設定
	void ini_set()
	{
		double p = rn.nextDouble() * 2 * (d.width + d.height);
		if (p < d.width) {   // 上
			x = r + rn.nextDouble() * (d.width - 2 * r);
			y = r;
		}
		else if (p < d.width + d.height) {   // 右
			x = d.width - r;
			y = r + rn.nextDouble() * (d.height - 2 * r);
		}
		else if (p < 2 * d.width + d.height) {   // 下
			x = r + rn.nextDouble() * (d.width - 2 * r);
			y = d.height - r;
		}
		else {   // 左
			x = r;
			y = r + rn.nextDouble() * (d.height - 2 * r);
		}
	}
					// 他ページへ移動の際，一時的にスレッドを停止
	public void stop()
	{
		state = false;
	}
					// スレッドの実行
	public void run()
	{
		while (state) {

			repaint();

			try {
				th.sleep(33);
			}
			catch (InterruptedException e) {}

			if (count == 0) {
				for (int i1 = 0; i1 < 15; i1++) {
					hit = cir.contains(px[i1], py[i1]);
					if (hit)
						break;
				}

				if (hit)
					count = 1;
				else {
					x += (vx * dt);
					y += (vy * dt);
				}
			}
			else {
				count++;
				if (count == 30) {
					count = 0;
					ini_set();
					double ang = Math.atan2(y-d.height/2, d.width/2-x);
					vx = v * Math.cos(ang);
					vy = -v * Math.sin(ang);
				}
			}
		}
	}
					// 描画
	public void paintComponent(Graphics g)
	{
		super.paintComponent(g);   // 親クラスの描画
							// Graphics2Dの取得
		Graphics2D g2 = (Graphics2D)g;
							// 塗りつぶした円
		g2.setColor(Color.green);
		cir = new Ellipse2D.Double(x-r, y-r, 40, 40);
		g2.fill(cir);
							// 塗りつぶした三角形
		g2.setColor(Color.red);
		g2.fill(tr);
	}
}
