import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import java.util.Random;

public class Test {
	public static void main (String[] args)
	{
		Win win = new Win("乱数（複数の円）");
	}
}

class Win extends JFrame
{
	/******************/
	/* コンストラクタ */
	/******************/
	Win(String name)
	{
					// JFrameクラスのコンストラクタ（Windowのタイトルを引き渡す）
		super(name);
					// Windowの大きさ
		setSize(640, 470);   // +40, +70
					// ContentPane の取得と MainPanel の追加
		Dimension d  = getSize();   // Windowの大きさ
		d.width     -= 40;
		d.height    -= 70;
		MainPanel pn = new MainPanel(d);   // MainPanel オブジェクトの生成
		getContentPane().add(pn);   // MainPanel オブジェクトを ContentPane に追加
					// ウィンドウを表示
		setVisible(true);
					// イベントアダプタ
		addWindowListener(new WinEnd());
	}

	/******************************/
	/* 上，左，下，右の余白の設定 */
	/******************************/
	public Insets getInsets()
	{
		return new Insets(50, 20, 20, 20);
	}

	/************/
	/* 終了処理 */
	/************/
	class WinEnd extends WindowAdapter
	{
		public void windowClosing(WindowEvent e) {
			System.exit(0);
		}
	}
}

class MainPanel extends JPanel implements Runnable
{
	boolean state = true;
	int n = 3;   // 物体の数
	double x[] = new double [n];   // 物体の位置(x)
	double y[] = new double [n];   // 物体の位置(y)
	double vx[] = new double [n];   // 物体の速度(x)
	double vy[] = new double [n];   // 物体の速度(y)
	double v = 30;   // 初期速度
	double dt = 0.1;   // 時間刻み幅
	int r = 20;   // 物体の半径
	Dimension d;   // パネルの幅と高さ
	Random rn;
	Thread th;

	MainPanel(Dimension d1)
	{
		d = d1;
		rn = new Random();
		for (int i1 = 0; i1 < n; i1++) {
			double p = rn.nextDouble() * 2 * (d.width + d.height);
			double ang;
			if (p < d.width) {   // 上
				ang   = -rn.nextDouble() * Math.PI;
				x[i1] = r + rn.nextDouble() * (d.width - 2 * r);
				y[i1] = r;
			}
			else if (p < d.width + d.height) {   // 右
				ang   = 0.5 * Math.PI + rn.nextDouble() * Math.PI;
				x[i1] = d.width - r;
				y[i1] = r + rn.nextDouble() * (d.height - 2 * r);
			}
			else if (p < 2 * d.width + d.height) {   // 下
				ang   = rn.nextDouble() * Math.PI;
				x[i1] = r + rn.nextDouble() * (d.width - 2 * r);
				y[i1] = d.height - r;
			}
			else {   // 左
				ang   = -0.5 * Math.PI + rn.nextDouble() * Math.PI;
				x[i1] = r;
				y[i1] = r + rn.nextDouble() * (d.height - 2 * r);
			}
			vx[i1] = v * Math.cos(ang);
			vy[i1] = -v * Math.sin(ang);
		}
		setBackground(new Color(238, 255, 238));   // 背景色の設定
		th = new Thread(this);   // スレッドの生成とスタート
		th.start();
	}
					// 他ページへ移動の際，一時的にスレッドを停止
	public void stop()
	{
		state = false;
	}
					// スレッドの実行
	public void run()
	{
		while (state) {
			try {
				th.sleep(33);
			}
			catch (InterruptedException e) {}

			for (int i1 = 0; i1 < n; i1++) {
				double a = 0.5 + 1.5 * rn.nextDouble();
				if (y[i1] + r > d.height) {   // 下の壁
					vx[i1] = a * vx[i1];
					vy[i1] = -a * vy[i1];
					y[i1]  = d.height - r;   // 位置の修正
				}
				else if (y[i1] - r < 0) {   // 上の壁
					vx[i1] = a * vx[i1];
					vy[i1] = -a * vy[i1];
					y[i1]  = r;   // 位置の修正
				}
				else if (x[i1] + r > d.width) {   // 右の壁
					vx[i1] = -a * vx[i1];
					vy[i1] = a * vy[i1];
					x[i1]  = d.width - r;   // 位置の修正
				}
				else if (x[i1] - r < 0) {   // 左の壁
					vx[i1] = -a * vx[i1];
					vy[i1] = a * vy[i1];
					x[i1]  = r;   // 位置の修正
				}
				x[i1] += vx[i1] * dt;
				y[i1] += vy[i1] * dt;
			}

			repaint();
		}
	}
					// 描画
	public void paintComponent(Graphics g)
	{
		super.paintComponent(g);   // 親クラスの描画

		g.setColor(Color.green);
		for (int i1 = 0; i1 < n; i1++)
			g.fillOval((int)x[i1]-r, (int)y[i1]-r, 2*r, 2*r);
	}
}
