import java.awt.*;
import java.awt.event.*;
import javax.swing.*;

public class Test {
	public static void main (String[] args)
	{
		Win win = new Win("等速度運動と等加速度運動");
	}
}

class Win extends JFrame
{
	/******************/
	/* コンストラクタ */
	/******************/
	Win(String name)
	{
					// JFrameクラスのコンストラクタ（Windowのタイトルを引き渡す）
		super(name);
					// Windowの大きさ
		setSize(640, 470);   // +40, +70
					// ContentPane の取得と MainPanel の追加
		Dimension d  = getSize();   // Windowの大きさ
		d.width     -= 40;
		d.height    -= 70;
		MainPanel pn = new MainPanel(d);   // MainPanel オブジェクトの生成
		getContentPane().add(pn);   // MainPanel オブジェクトを ContentPane に追加
					// ウィンドウを表示
		setVisible(true);
					// イベントアダプタ
		addWindowListener(new WinEnd());
	}

	/******************************/
	/* 上，左，下，右の余白の設定 */
	/******************************/
	public Insets getInsets()
	{
		return new Insets(50, 20, 20, 20);
	}

	/************/
	/* 終了処理 */
	/************/
	class WinEnd extends WindowAdapter
	{
		public void windowClosing(WindowEvent e) {
			System.exit(0);
		}
	}
}

class MainPanel extends JPanel implements Runnable
{
	boolean state = true;
	double x0 = 0.0, x1, x2, y1, y2;   // x方向の初期位置と各物体の位置
	double t = 0.0, dt = 0.04;   // 時間とその刻み幅
	double v0 = 50.0, v1, v2;   // 初期速度と各物体の速度
	double a = 30.0;   // 加速度
	double ang1 = 20.0 * Math.PI / 180.0, ang2 = -20.0 * Math.PI / 180.0;   // 進行方向
	Dimension d;   // パネルの幅と高さ
	Thread th;

	MainPanel(Dimension d1)
	{
		d  = d1;
		x1 = x0;
		y1 = d.height / 2;
		v1 = v0;
		x2 = x0;
		y2 = d.height / 2;
		v2 = v0;
		setBackground(new Color(238, 255, 238));   // 背景色の設定
		th = new Thread(this);   // スレッドの生成とスタート
		th.start();
	}
					// 他ページへ移動の際，一時的にスレッドを停止
	public void stop()
	{
		state = false;
	}
					// スレッドの実行
	public void run()
	{
		while (state) {
			try {
				th.sleep(40);
			}
			catch (InterruptedException e) {}
			if (x2 < d.width) {
				x1 += v1 * dt * Math.cos(ang1);
				y1 += -v1 * dt * Math.sin(ang1);
				double t1 = v2;
				double t2 = v2 + a * dt;
				v2  = t2;
				x2 += 0.5 * (t1 + t2) * dt * Math.cos(ang2);
				y2 += -0.5 * (t1 + t2) * dt * Math.sin(ang2);
			}
			else {
				x1 = x0;
				v1 = v0;
				y1 = d.height / 2;
				x2 = x0;
				v2 = v0;
				y2 = d.height / 2;
				t  = 0;
			};
			repaint();
		}
	}
					// 描画
	public void paintComponent(Graphics g)
	{
		super.paintComponent(g);   // 親クラスの描画

		g.setColor(Color.green);
		g.fillOval((int)x1-20, (int)y1-20, 40, 40);
		g.setColor(Color.red);
		g.fillOval((int)x2-20, (int)y2-20, 40, 40);
	}
}
