import java.awt.*;
import java.awt.event.*;
import javax.swing.*;

public class Test {
	public static void main (String[] args)
	{
		Win win = new Win("摩擦３");
	}
}

class Win extends JFrame
{
	/******************/
	/* コンストラクタ */
	/******************/
	Win(String name)
	{
					// JFrameクラスのコンストラクタ（Windowのタイトルを引き渡す）
		super(name);
					// Windowの大きさ
		setSize(640, 470);   // +40, +70
					// ContentPane の取得と MainPanel の追加
		Dimension d  = getSize();   // Windowの大きさ
		d.width     -= 40;
		d.height    -= 70;
		MainPanel pn = new MainPanel(d);   // MainPanel オブジェクトの生成
		getContentPane().add(pn);   // MainPanel オブジェクトを ContentPane に追加
					// ウィンドウを表示
		setVisible(true);
					// イベントアダプタ
		addWindowListener(new WinEnd());
	}

	/******************************/
	/* 上，左，下，右の余白の設定 */
	/******************************/
	public Insets getInsets()
	{
		return new Insets(50, 20, 20, 20);
	}

	/************/
	/* 終了処理 */
	/************/
	class WinEnd extends WindowAdapter
	{
		public void windowClosing(WindowEvent e) {
			System.exit(0);
		}
	}
}

class MainPanel extends JPanel implements Runnable
{
	boolean state = true;
	double dt = 0.04;
	double x11 = 0.0, x12 = 0.0, y1 = 0.0, v11 = 100.0, v12 = 100.0;
	double x21 = 0.0, x22 = 0.0, y2 = 0.0, v21 = 0.0, v22 = 0.0;
	double a = 200.0;   // 加速度
	double f1 = 0.2, f2 = 0.7;   // 摩擦係数
	Dimension d;
	Thread th;

	MainPanel(Dimension d1)
	{
		d  = d1;
		y1 = d.height / 2 - 100;
		y2 = d.height / 2 + 100;
		setBackground(new Color(238, 255, 238));   // 背景色の設定
		th = new Thread(this);   // スレッドの生成とスタート
		th.start();
	}
					// 他ページへ移動の際，一時的にスレッドを停止
	public void stop()
	{
		state = false;
	}
					// スレッドの実行
	public void run()
	{
		while (state) {
			try {
				th.sleep(33);
			}
			catch (InterruptedException e) {}
			if (x11 < d.width) {
							// 摩擦無し
									// 等速直線運動
				x11 += v11 * dt;
									// 等加速度運動
				double v = v21 + a * dt;   // 加速度 a による速度の変化（加速）
				x21 += 0.5 * (v21 + v) * dt;   // 加速前と加速後の速度の平均で位置を計算
				v21  = v;   // 加速前と加速後の速度の平均で位置を計算
							// 摩擦あり
									// 等速直線運動＋摩擦
				v    = v12 - f1 * v12 * dt;   // 速度に比例した摩擦による速度の変化（減速）
				x12 += 0.5 * (v12 + v) * dt;   // 減速前と減速後の速度の平均で位置を計算
				v12  = v;   // 加速前と加速後の速度の平均で位置を計算
									// 等加速度運動＋摩擦
				v = v22 + (a - f2 * v22) * dt;   // 本来の加速度と速度に比例した摩擦による速度の変化
				x22 += 0.5 * (v22 + v) * dt;   // 変化前と変化後の速度の平均で位置を計算
				v22  = v;   // 加速前と加速後の速度の平均で位置を計算
			}
			else {
				x11 = 0.0;
				x12 = 0.0;
				v12 = 100.0;
				x21 = 0.0;
				x22 = 0.0;
				v21 = 0.0;
				v22 = 0.0;
			}
			repaint();
		}
	}
					// 描画
	public void paintComponent(Graphics g)
	{
		super.paintComponent(g);   // 親クラスの描画
							// 摩擦無し
		g.setColor(Color.green);
		g.fillOval((int)x11-20, (int)y1-20, 40, 40);
		g.fillOval((int)x21-20, (int)y2-20, 40, 40);
							// 摩擦あり
		g.setColor(Color.red);
		g.fillOval((int)x12-20, (int)y1-20, 40, 40);
		g.fillOval((int)x22-20, (int)y2-20, 40, 40);
	}
}
