import java.awt.*;
import java.awt.event.*;
import javax.swing.*;

public class Test {
	public static void main (String[] args)
	{
		Win win = new Win("周期運動１");
	}
}

class Win extends JFrame
{
	/******************/
	/* コンストラクタ */
	/******************/
	Win(String name)
	{
					// JFrameクラスのコンストラクタ（Windowのタイトルを引き渡す）
		super(name);
					// Windowの大きさ
		setSize(440, 670);   // +40, +70
					// ContentPane の取得と MainPanel の追加
		Dimension d  = getSize();   // Windowの大きさ
		d.width     -= 40;
		d.height    -= 70;
		MainPanel pn = new MainPanel(d);   // MainPanel オブジェクトの生成
		getContentPane().add(pn);   // MainPanel オブジェクトを ContentPane に追加
					// ウィンドウを表示
		setVisible(true);
					// イベントアダプタ
		addWindowListener(new WinEnd());
	}

	/******************************/
	/* 上，左，下，右の余白の設定 */
	/******************************/
	public Insets getInsets()
	{
		return new Insets(50, 20, 20, 20);
	}

	/************/
	/* 終了処理 */
	/************/
	class WinEnd extends WindowAdapter
	{
		public void windowClosing(WindowEvent e) {
			System.exit(0);
		}
	}
}

class MainPanel extends JPanel implements Runnable
{
	boolean state = true;
	double g = 9.8, m = 1.0, r = 0.5, I = 4.0 * r * r / 12.0;
	double a20 = 20.0 * Math.PI / 180.0;
	double k1 = 0, k2 = 0.5, k3 = 2.6;
	double p1 = k1 / (I + m * r * r), p2 = k2 / (I + m * r * r), p3 = k3 / (I + m * r * r);
	double q = m * r * g / (I + m * r * r);
	double alpha1 = -0.5 * p1, beta1 = 0.5 * Math.sqrt(4 * q - p1 * p1);
	double c12 = 0.5 * Math.PI, c11 = a20;
	double alpha2 = -0.5 * p2, beta2 = 0.5 * Math.sqrt(4 * q - p2 * p2);
	double c22 = Math.atan(-beta2 / alpha2), c21 = a20 / Math.sin(c22);
	double m1 = 0.5 * (-p3 + Math.sqrt(p3 * p3 - 4 * q));
	double m2 = 0.5 * (-p3 - Math.sqrt(p3 * p3 - 4 * q));
	double c31 = -a20 * m2 / (m1 - m2), c32 = a20 - c31;
	double len, t = 0, dt = 0.01, x1, x2, x3, y1, y2, y3;
	Dimension d;
	Thread th;

	MainPanel(Dimension d1)
	{
		d = d1;
		len = d.height - 100;
		setBackground(new Color(238, 255, 238));   // 背景色の設定

		double z = Math.sin(a20);
		x1 = 200 + len * Math.sin(z);
		y1 = 50 + len * Math.cos(z);
		x2 = x1;
		y2 = y1;
		x3 = x1;
		y3 = y1;

		th = new Thread(this);   // スレッドの生成とスタート
		th.start();
	}
					// 他ページへ移動の際，一時的にスレッドを停止
	public void stop()
	{
		state = false;
	}
					// スレッドの実行
	public void run()
	{
		while (state) {
			try {
				th.sleep(40);
			}
			catch (InterruptedException e) {}

			t += dt;

			double z = c11 * Math.exp(alpha1 * t) * Math.sin(beta1 * t + c12);
			x1 = 200 + len * Math.sin(z);
			y1 = 50 + len * Math.cos(z);

			z = c21 * Math.exp(alpha2 * t) * Math.sin(beta2 * t + c22);
			x2 = 200 + len * Math.sin(z);
			y2 = 50 + len * Math.cos(z);

			z = c31 * Math.exp(m1 * t) + c32 * Math.exp(m2 * t);
			x3 = 200 + len * Math.sin(z);
			y3 = 50 + len * Math.cos(z);

			repaint();
		}
	}
					// 描画
	public void paintComponent(Graphics g)
	{
		super.paintComponent(g);   // 親クラスの描画
							// 軸
		g.setColor(Color.black);
		g.fillOval(195, 45, 10, 10);
							// Graphics2Dの取得
		Graphics2D g2 = (Graphics2D)g;
							// 摩擦無し
		g.setColor(Color.green);
		g2.setStroke(new BasicStroke(3.0f));
		g2.drawLine(200, 50, (int)x1, (int)y1);
							// 摩擦小，k = 0.5
		g.setColor(Color.red);
		g2.drawLine(200, 50, (int)x2, (int)y2);
							// 摩擦大，k = 2.6
		g.setColor(Color.blue);
		g2.drawLine(200, 50, (int)x3, (int)y3);
	}
}
