import java.awt.*;
import java.awt.event.*;
import javax.swing.*;

public class Test {
	public static void main (String[] args)
	{
		Win win = new Win("周期運動２");
	}
}

class Win extends JFrame
{
	/******************/
	/* コンストラクタ */
	/******************/
	Win(String name)
	{
					// JFrameクラスのコンストラクタ（Windowのタイトルを引き渡す）
		super(name);
					// Windowの大きさ
		setSize(840, 670);   // +40, +70
					// ContentPane の取得と MainPanel の追加
		Dimension d  = getSize();   // Windowの大きさ
		d.width     -= 40;
		d.height    -= 70;
		MainPanel pn = new MainPanel(d);   // MainPanel オブジェクトの生成
		getContentPane().add(pn);   // MainPanel オブジェクトを ContentPane に追加
					// ウィンドウを表示
		setVisible(true);
					// イベントアダプタ
		addWindowListener(new WinEnd());
	}

	/******************************/
	/* 上，左，下，右の余白の設定 */
	/******************************/
	public Insets getInsets()
	{
		return new Insets(50, 20, 20, 20);
	}

	/************/
	/* 終了処理 */
	/************/
	class WinEnd extends WindowAdapter
	{
		public void windowClosing(WindowEvent e) {
			System.exit(0);
		}
	}
}

class MainPanel extends JPanel implements Runnable
{
	boolean state = true;
	double g = 9.8, m = 1.0, r = 0.5, I = 4.0 * r * r / 12.0, a20 = 20.0;
	double k1 = 0, k2 = 0.5, k3 = 2.6;
	double p1 = k1 / (I + m * r * r), p2 = k2 / (I + m * r * r), p3 = k3 / (I + m * r * r);
	double q = m * r * g / (I + m * r * r);
	double alpha1 = -0.5 * p1, beta1 = 0.5 * Math.sqrt(4 * q - p1 * p1);
	double c12 = 0.5 * Math.PI, c11 = a20;
	double alpha2 = -0.5 * p2, beta2 = 0.5 * Math.sqrt(4 * q - p2 * p2);
	double c22 = Math.atan(-beta2 / alpha2), c21 = a20 / Math.sin(c22);
	double m1 = 0.5 * (-p3 + Math.sqrt(p3 * p3 - 4 * q));
	double m2 = 0.5 * (-p3 - Math.sqrt(p3 * p3 - 4 * q));
	double c31 = -a20 * m2 / (m1 - m2), c32 = a20 - c31;
	double amp, t = 0, dt = 0.01, x, y1, y2, y3;
	Dimension d;
	Thread th;

	MainPanel(Dimension d1)
	{
		d = d1;
		amp = (d.height / 2 - 20) / a20;
		setBackground(new Color(238, 255, 238));   // 背景色の設定

		x = 0;
		y1 = d.height / 2 - a20 * amp;
		y2 = y1;
		y3 = y1;

		th = new Thread(this);   // スレッドの生成とスタート
		th.start();
	}
					// 他ページへ移動の際，一時的にスレッドを停止
	public void stop()
	{
		state = false;
	}
					// スレッドの実行
	public void run()
	{
		while (state) {
			try {
				th.sleep(40);
			}
			catch (InterruptedException e) {}

			t += dt;

			x = 200 * t;
			y1 = d.height / 2 - amp * c11 * Math.exp(alpha1 * t) * Math.sin(beta1 * t + c12);
			y2 = d.height / 2 - amp * c21 * Math.exp(alpha2 * t) * Math.sin(beta2 * t + c22);
			y3 = d.height / 2 - amp * (c31 * Math.exp(m1 * t) + c32 * Math.exp(m2 * t));

			repaint();
		}
	}
					// 描画
	public void paintComponent(Graphics g)
	{
		super.paintComponent(g);   // 親クラスの描画
							// Graphics2Dの取得
		Graphics2D g2 = (Graphics2D)g;
							// 軸
		g.setColor(Color.black);
		g2.setStroke(new BasicStroke(2.0f));
		g2.drawLine(0, d.height/2, d.width, d.height/2);
		g.drawString("時間", d.width-30, d.height/2+15);

		g.setColor(new Color(0xc0, 0xc0, 0xc0));
		g2.drawLine(0, 20, d.width, 20);
		g.setColor(Color.black);
		g.drawString("θ=20度", 5, 15);

		g.setColor(new Color(0xc0, 0xc0, 0xc0));
		g2.drawLine(0, d.height-20, d.width, d.height-20);
		g.setColor(Color.black);
		g.drawString("θ=-20度", 5, d.height-5);

		if (x < d.width) {
							// 摩擦無し
			g.setColor(Color.green);
			g.fillOval((int)x-10, (int)y1-10, 20, 20);
							// 摩擦小，k = 0.5
			g.setColor(Color.red);
			g.fillOval((int)x-10, (int)y2-10, 20, 20);
							// 摩擦大，k = 2.6
			g.setColor(Color.blue);
			g.fillOval((int)x-10, (int)y3-10, 20, 20);
		}
		else {
			t = 0.0;
			x = 0.0;
			y1 = d.height / 2 - a20 * amp;
			y2 = y1;
			y3 = y1;
		}
	}
}
