package game;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import java.util.Random;
import main.*;

public class GamePanel extends JPanel implements ActionListener, Runnable
{
	Dimension size;   // パネルの大きさ
	MainPanel mp;
	JButton left, right;
	Block blk;   // ブロック
	Racket rk;   // ラケット
	Ball bl;   // ボール
	Random rand;
	Thread td;
	boolean in_game = true;
			// コンストラクタ
	public GamePanel(Dimension size1, MainPanel mp1)
	{
		size = size1;
		mp   = mp1;
					// レイアウトマネージャの停止
		setLayout(null);
					// 背景色の設定
		setBackground(Color.white);
					// ブロックの配置
		blk = new Block(mp);
					// ボタンの配置
		Font f = new Font("SansSerif", Font.BOLD, 15);
		FontMetrics fm = getFontMetrics(f);
		String str1 = "左";
		int w1 = fm.stringWidth(str1) + 40;
		int h1 = fm.getHeight() + 10;
		left = new JButton(str1);
		left.setFont(f);
		left.addActionListener(this);   // アクションリスナ
		left.setSize(w1, h1);

		String str2 = "右";
		int w2 = fm.stringWidth(str2) + 40;
		int h2 = fm.getHeight() + 10;
		right = new JButton(str2);
		right.setFont(f);
		right.addActionListener(this);   // アクションリスナ
		right.setSize(w2, h2);

		int w = size.width / 2 - (w1 + w2 + 5) / 2;
		int h = size.height - h1 - 10;
		left.setLocation(w, h);
		add(left);
		right.setLocation(w+w1+5, h);
		add(right);
					// ランダム変数の初期化
		rand = new Random();
					// ボール及びラケットの配置
		rk = new Racket(h-10, size, mp);
		bl = new Ball(blk, size, rand);
					// スレッドの生成
		td = new Thread(this);
		td.start();
	}
			// スレッドの実行
	public void run()
	{
		while (in_game) {
			try {
				td.sleep(50);
			}
			catch (InterruptedException e) {}
					// 移動
			bl.x += bl.vx;
			bl.y += bl.vy;
					// 壁に衝突したときの処理
			int sw = 0;
							// 下へ移動中
			if (bl.vy > 0) {
				if (bl.y >= size.height-bl.r*2) {   // 下の壁に衝突
					bl.y  = size.height - bl.r * 2;
					bl.vy = -bl.vy;
					sw    = 1;
				}
			}
							// 上へ移動中
			else {
				if (bl.y <= 0) {   // 上の壁に衝突
					bl.y  = 0;
					bl.vy = -bl.vy;
					sw    = 1;
				}
			}
							// 上下の壁に衝突していない場合
			if (sw == 0) {
									// 右方向へ移動中
				if (bl.vx > 0) {
					if (bl.x >= size.width-bl.r*2) {   // 右の壁に衝突
						bl.x  = size.width - bl.r * 2;
						bl.vx = -bl.vx;
					}
				}
									// 左方向へ移動中
				else {
					if (bl.x <= 0) {   // 左の壁に衝突
						bl.x  = 0;
						bl.vx = -bl.vx;
					}
				}
			}
					// 再描画
			repaint();
		}
	}
			// 描画
	public void paintComponent(Graphics g)
	{
		super.paintComponent(g);   // 親クラスの描画
					// ブロック
		for (int i1 = 0; i1 < blk.row; i1++) {
			for (int i2 = 0; i2 < blk.col; i2++) {
				if (blk.ex[i1][i2])
					g.drawImage(blk.block, blk.width*i2, blk.height*i1, this);
			}
		}
					// ラケット
		g.setColor(rk.color);
		g.fillRect(rk.x, rk.y, rk.width, rk.height);
					// ボール
		g.setColor(bl.color);
		g.fillOval(bl.x, bl.y, bl.r*2, bl.r*2);
	}
			// ボタンがクリックされたときの処理
	public void actionPerformed(ActionEvent e)
	{
		if (e.getSource() == left)   // ラケットを左へ移動
			rk.x -= (rk.width - 5);
		else   // ラケットを右へ移動
			rk.x += (rk.width - 5);
	}
}
