package game;

import java.awt.*;
import javax.swing.*;
import java.util.Random;
import main.*;

class Puyo extends JPanel implements Runnable
{
	MainPanel mp;
	int p_x, p_y;   // 左または上のピースの座標
	int row, col;   // ゲーム画面の行と列数
	int width;   // ピースの幅と高さ
	int rot;   // 0:横，1:縦
	int p[][];   // 画面の状態（0:存在しない，1:赤，2:ピンク，3:緑，4:青）
	Color color[] = {Color.red, Color.pink, Color.green, Color.blue};   // ピースの色
	int pp[][];   // ワーク
	int d_no = 4;   // 同じ色のピースがd_no以上連続すると消去
	boolean ok = true;   // 回転，色の交換が可能か？
	Thread th;
	boolean in_game = true;
	Random rn;
			// コンストラクタ
	public Puyo(int row1, int col1, int width1, MainPanel mp1)
	{
		mp    = mp1;
		row   = row1;
		col   = col1;
		width = width1;
		p     = new int [row][col];
		pp    = new int [row][col];
					// レイアウトマネージャの停止
		setLayout(null);
					// 背景色の設定
		setBackground(Color.white);
					// ランダム変数の初期化
		rn = new Random();
					// ピースの選択
		select();
					// スレッドの生成
		th = new Thread(this);
		th.start();
	}
			// スレッドの実行
	public void run()
	{
		while (in_game) {
			try {
				th.sleep(500);
			}
			catch (InterruptedException e) {}
					// ピースの落下
			int ct = 0;
							// 横
			if (rot == 0) {
				if (p_y < row-1) {
					if (ok) {
						if (p[p_y+1][p_x] == 0 && p[p_y+1][p_x+1] == 0) {
							ct              = 1;
							p[p_y+1][p_x]   = p[p_y][p_x];
							p[p_y+1][p_x+1] = p[p_y][p_x+1];
							p[p_y][p_x]     = 0;
							p[p_y][p_x+1]   = 0;
							p_y++;
						}
						else {
							ok = false;
							if (p[p_y+1][p_x] == 0) {
								ct            = 1;
								p[p_y+1][p_x] = p[p_y][p_x];
								p[p_y][p_x]   = 0;
								p_y++;
							}
							else if (p[p_y+1][p_x+1] == 0) {
								ct              = 1;
								p[p_y+1][p_x+1] = p[p_y][p_x+1];
								p[p_y][p_x+1]   = 0;
								p_x++;
								p_y++;
							}
						}
					}
					else {
						if (p[p_y+1][p_x] == 0) {
							ct            = 1;
							p[p_y+1][p_x] = p[p_y][p_x];
							p[p_y][p_x]   = 0;
							p_y++;
						}
					}
				}
				else
					ok = false;
			}
							// 縦
			else {
				if (p_y < row-2 && p[p_y+2][p_x] == 0) {
					ct = 1;
					p[p_y+2][p_x] = p[p_y+1][p_x];
					p[p_y+1][p_x] = p[p_y][p_x];
					p[p_y][p_x]   = 0;
					p_y++;
				}
				else
					ok = false;
			}
					// 消去と次のピース
			if (ct == 0) {   // 落下できなかった場合
				ct = d_no;
				while (ct >= d_no) {
					ct = 0;
					for (int i1 = row-1; i1 >= 0 && ct < d_no; i1--) {
						for (int i2 = 0; i2 < col && ct < d_no; i2++) {
							if (p[i1][i2] > 0) {
								for (int i3 = 0; i3 < row; i3++) {
									for (int i4 = 0; i4 < col; i4++)
										pp[i3][i4] = 0;
								}
								pp[i1][i2] = 1;
								ct         = search(i1, i2, 1);
							}
						}
					}
					if (ct >= d_no)
						del();
				}
				select();
			}
					// 再描画
			repaint();
		}
	}
			// 描画
	public void paintComponent(Graphics g)
	{
		super.paintComponent(g);   // 親クラスの描画
		for (int i1 = 0; i1 < row; i1++) {
			for (int i2 = 0; i2 < col; i2++) {
				if (p[i1][i2] > 0) {
					g.setColor(color[p[i1][i2]-1]);
					g.fillRect(i2*width, i1*width, width, width);
				}
			}
		}
	}
			// ピースの選択
	void select()
	{
		ok  = true;
		rot = 0;
		p_y = 0;
		p_x = rn.nextInt(col - 1);
		if (p[0][p_x] == 0 && p[0][p_x+1] == 0) {
			p[0][p_x]   = 1 + rn.nextInt(4);
			p[0][p_x+1] = 1 + rn.nextInt(4);
		}
		else {   // ゲームオーバー
			in_game  = false;
			mp.state = 2;
		}
	}
			// 同じ色のピースを探す
			//      k1,k2 : 対象としているピース
			//      c1 : 同じ色のピースの数
			//      return : 同じ色のピースの数
	int search(int k1, int k2, int c1)
	{
		int ct = c1;

		if (k1 > 0 && p[k1-1][k2] == p[k1][k2] && pp[k1-1][k2] == 0) {
			pp[k1-1][k2] = 1;
			ct           = search(k1-1, k2, ct+1);
		}
		if (k1 < row-1 && p[k1+1][k2] == p[k1][k2] && pp[k1+1][k2] == 0) {
			pp[k1+1][k2] = 1;
			ct           = search(k1+1, k2, ct+1);
		}
		if (k2 > 0 && p[k1][k2-1] == p[k1][k2] && pp[k1][k2-1] == 0) {
			pp[k1][k2-1] = 1;
			ct           = search(k1, k2-1, ct+1);
		}
		if (k2 < col-1 && p[k1][k2+1] == p[k1][k2] && pp[k1][k2+1] == 0) {
			pp[k1][k2+1] = 1;
			ct           = search(k1, k2+1, ct+1);
		}

		return ct;
	}
			// 同じ色のピースを削除
	void del()
	{
		int i1, i2, i3, k1, k2, k3;
					// 削除
		for (i1 = 0; i1 < row; i1++) {
			for (i2 = 0; i2 < col; i2++) {
				if (pp[i1][i2]  > 0)
					p[i1][i2] = 0;
			}
		}
					// 詰める
		for (i1 = 0; i1 < col; i1++) {
			k1 = 1;
			for (i2 = row-1; i2 > 0 && k1 >= 0; i2--) {
				if (p[i2][i1] == 0) {
					k1 = -1;
					for (i3 = i2-1; i3 >= 0 && k1 < 0; i3--) {
						if (p[i3][i1] > 0)
							k1 = i3;
					}
					if (k1 >= 0) {
						k2 = i2;
						k3 = k2 - k1;
						while (k1 >= 0) {
							p[k2][i1] = p[k1][i1];
							k1--;
							k2--;
						}
						k1++;
						for (i3 = 0; i3 < k3; i3++)
							p[i3][i1] = 0;
					}
				}
			}
		}
	}
}
